/*
 * I2C support for the JZ4780.
 *
 * Copyright (C) 2017, 2018, 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/i2c-common.h>

// I2C channel.

class I2c_jz4780_channel : public I2c_channel
{
public:
  explicit I2c_jz4780_channel(l4_addr_t start, enum Clock_identifiers clock,
                              Cpm_chip *cpm, uint32_t frequency);
};

// I2C device control.

class I2c_jz4780_chip : public I2c_chip
{
protected:
  unsigned int num_channels()
  { return 5; }

public:
  explicit I2c_jz4780_chip(l4_addr_t start, l4_addr_t end, Cpm_chip *cpm,
                           uint32_t frequency);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4780_i2c_init(l4_addr_t start, l4_addr_t end, void *cpm,
                      uint32_t frequency);

void *jz4780_i2c_get_channel(void *i2c, uint8_t channel);

uint32_t jz4780_i2c_get_frequency(void *i2c_channel);

void jz4780_i2c_set_target(void *i2c_channel, uint8_t addr);

void jz4780_i2c_start_read(void *i2c_channel, uint8_t buf[], unsigned int total,
                          int stop);

void jz4780_i2c_read(void *i2c_channel);

void jz4780_i2c_start_write(void *i2c_channel, uint8_t buf[], unsigned int total,
                           int stop);

void jz4780_i2c_write(void *i2c_channel);

int jz4780_i2c_read_done(void *i2c_channel);

int jz4780_i2c_write_done(void *i2c_channel);

unsigned int jz4780_i2c_have_read(void *i2c_channel);

unsigned int jz4780_i2c_have_written(void *i2c_channel);

int jz4780_i2c_failed(void *i2c_channel);

EXTERN_C_END
