/*
 * I2C support for the JZ4780.
 *
 * Copyright (C) 2017, 2018, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/i2c-jz4780.h>

/*
I2C pins:

I2C0: PD30/SMB0_SDA,        PD31/SMB0_SCK
I2C1: PD30/SMB1_SDA,        PD31/SMB1_SCK (pulled up on CI20)
I2C2: PF16/SMB2_SDA,        PF17/SMB2_SCK
 PWM: PD10/SMB3_SDA,        PD11/SMB3_SCK
 PWM:  PE3/SMB4_SDA,         PE4/SMB4_SCK
I2C4: PE12/SMB4_SDA,        PE13/SMB4_SCK
HDMI: PF25/SMB4_SDA/DDCSDA, PF24/SMB4_SCK/DDCSCK

See: http://mipscreator.imgtec.com/CI20/hardware/board/ci20_jz4780_v2.0.pdf
*/



// Initialise a channel.

I2c_jz4780_channel::I2c_jz4780_channel(l4_addr_t start,
                                       enum Clock_identifiers clock,
                                       Cpm_chip *cpm,
                                       uint32_t frequency)
: I2c_channel(start, clock, cpm, frequency)
{
}



// Initialise the I2C controller.

I2c_jz4780_chip::I2c_jz4780_chip(l4_addr_t start, l4_addr_t end,
                                 Cpm_chip *cpm,
                                 uint32_t frequency)
: I2c_chip(start, end, cpm, frequency)
{
}



// C language interface functions.

void *jz4780_i2c_init(l4_addr_t start, l4_addr_t end, void *cpm, uint32_t frequency)
{
  return (void *) new I2c_jz4780_chip(start, end, static_cast<Cpm_chip *>(cpm), frequency);
}

void *jz4780_i2c_get_channel(void *i2c, uint8_t channel)
{
  return static_cast<I2c_jz4780_chip *>(i2c)->get_channel(channel);
}

uint32_t jz4780_i2c_get_frequency(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->get_frequency();
}

void jz4780_i2c_set_target(void *i2c_channel, uint8_t addr)
{
  static_cast<I2c_jz4780_channel *>(i2c_channel)->set_target(addr);
}

void jz4780_i2c_start_read(void *i2c_channel, uint8_t buf[], unsigned int total,
                          int stop)
{
  static_cast<I2c_jz4780_channel *>(i2c_channel)->start_read(buf, total, stop);
}

void jz4780_i2c_read(void *i2c_channel)
{
  static_cast<I2c_jz4780_channel *>(i2c_channel)->read();
}

void jz4780_i2c_start_write(void *i2c_channel, uint8_t buf[], unsigned int total,
                           int stop)
{
  static_cast<I2c_jz4780_channel *>(i2c_channel)->start_write(buf, total, stop);
}

void jz4780_i2c_write(void *i2c_channel)
{
  static_cast<I2c_jz4780_channel *>(i2c_channel)->write();
}

int jz4780_i2c_read_done(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->read_done();
}

int jz4780_i2c_write_done(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->write_done();
}

unsigned int jz4780_i2c_have_read(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->have_read();
}

unsigned int jz4780_i2c_have_written(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->have_written();
}

int jz4780_i2c_failed(void *i2c_channel)
{
  return static_cast<I2c_jz4780_channel *>(i2c_channel)->failed();
}
